#!/usr/bin/env python
#
# CVS mail attaching filter (for python 2.2.2 and later)
#
# Copyright(C) 2002-2003 Hye-Shik Chang <perky@FreeBSD.org>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
# IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#

import sys, os
import email, urllib
import re
from cStringIO import StringIO

HOME = os.environ['HOME']
LOGFILE = os.sep.join([HOME, '.cvsmailattach.log'])
CVSWEBDIFFURL = ('http://www.freebsd.org/cgi/cvsweb.cgi/%(path)s.diff?'
                 'r1=%(r1)s&r2=%(r2)s')
CVSWEBGETURL = ('http://www.freebsd.org/cgi/cvsweb.cgi/~checkout~/%(path)s?'
                'rev=%(rev)s&content-type=text/plain')

re_revisions_start = re.compile('^\s+Revision\s+Changes\s+Path')
re_changeline = re.compile('([0-9]+(\.[0-9]+)+)\s+\+([0-9]+)\s+-([0-9]+)\s+'
                           '([^\n\r\t ]+)(\s+\((dead|new)\))?')

def findmodfiles(mail):
    modfiles = []
    status_in = False
    for line in mail.get_payload().splitlines():
        if status_in:
            if re_changeline.search(line):
                revision, ignore, added, removed, path, ignore, newdead = \
                re_changeline.findall(line)[0]
                modfiles.append((path, revision, newdead,
                                 int(added), int(removed)))
            else:
                status_in = False
        else:
            if re_revisions_start.search(line):
                status_in = True
    return modfiles

def getprevrev(rev):
    frags = rev.split('.')
    frags[-1] = str(int(frags[-1]) - 1)
    if frags[-1] == '0':
        frags = frags[:-2] # branched
    return '.'.join(frags)

def filt(outf, inf):
    inf = StringIO(inf.read())
    try:
        mail = email.message_from_file(inf)

        branch = mail.get('X-FreeBSD-CVS-Branch')
        if not branch:
            raise ValueError, "Doesn't have branch header"

        modfiles = findmodfiles(mail)
        if not modfiles:
            raise ValueError, "No file is modified, eh?"

        urls = []
        for path, revision, newdead, added, removed in modfiles:
            if newdead == 'new':
                urls.append((path,
                            CVSWEBGETURL % {'path': path, 'rev': revision}))
            elif newdead == 'dead':
                pass
            else:
                urls.append((path,
                            CVSWEBDIFFURL % {'path': path,
                            'r1': getprevrev(revision), 'r2': revision}))

        newmail = email.message_from_string(inf.getvalue())
        newmail['Content-Type'] = 'multipart/mixed'
        newmail.set_payload([])
        newmail.attach(email.message_from_string(
                    "Content-type: text/plain\n\n" + mail.get_payload()))

        for path, url in urls:
            filecontent = urllib.urlopen(url).read()
            newmail.attach(email.message_from_string(
                "Content-type: text/plain; name=\"%(name)s\"\n"
                "Content-Disposition: attachment; filename=\"%(name)s\"\n\n"
                "%(content)s"
                % {'name': path, 'content': filecontent}))

        print >> outf, newmail

    except:
        import traceback
        logf = open(LOGFILE, 'a')
        print >> logf, "="*78
        traceback.print_exc(file=logf)
        print >> logf, inf.getvalue()
        print >> logf, "="*78
        del logf
        sys.exit(-1)

if __name__ == '__main__':
    if len(sys.argv) > 1:
        for f in sys.argv[1:]:
            filt(sys.stdout, open(f))
    else:
        filt(sys.stdout, sys.stdin)

# vim: ts=4 sts=4 et sw=4
